@@ -1,14 +1,18 @@
-from typing import Optional, Type
-
 from concurrent import futures
+from typing import Optional, Type
 
 import grpc
 import oaas
 import oaas._registrations as registrations
+from oaas_grpc import OaasGrpcClient
 from oaas_grpc.server.find_ips import find_ips
 from oaas_registry_api.rpc.registry_pb2 import OaasServiceDefinition
 from oaas_registry_api.rpc.registry_pb2_grpc import OaasRegistryStub
 
+import logging
+
+LOG = logging.getLogger(__name__)
+
 
 def find_add_to_server_base(t: Type) -> Optional[Type]:
     items_to_process = {t}
@@ -35,6 +39,8 @@ class OaasGrpcServer(oaas.ServerMiddleware):
 
         self._is_serving = True
 
+        ensure_grpc_client()
+
         server_address: str = f"[::]:{self.port}"
         self.server = grpc.server(futures.ThreadPoolExecutor())
 
@@ -44,8 +50,10 @@ class OaasGrpcServer(oaas.ServerMiddleware):
             if not self.can_serve(service_definition):
                 continue
 
-            print(
-                f"Added GRPC service: {service_definition.gav} as {service_definition.code}"
+            LOG.info(
+                "Added GRPC service: %s as %s",
+                service_definition.gav,
+                service_definition.code,
             )
 
             find_add_to_server_base(service_definition.code).add_to_server(  # type: ignore
@@ -56,7 +64,7 @@ class OaasGrpcServer(oaas.ServerMiddleware):
 
         locations = find_ips(port=self.port)
 
-        print(f"listening on {port}")
+        LOG.info("listening on %d", port)
         self.server.start()
 
         # we register the services
@@ -77,3 +85,17 @@ class OaasGrpcServer(oaas.ServerMiddleware):
 
     def can_serve(self, service_definition: oaas.ServiceDefinition) -> bool:
         return find_add_to_server_base(service_definition.code) is not None
+
+
+def ensure_grpc_client():
+    """
+    If we're exposing gRPC services, we also need to register them
+    into the oaas-registry. For this to happen, we need to have the
+    OaasGrpcClient registered. If the user forgot to add it, we'll
+    add it ourselves.
+    """
+    for client in registrations.clients_middleware:
+        if isinstance(client, OaasGrpcClient):
+            return
+
+    oaas.register_client_provider(OaasGrpcClient())
